<?php

namespace core\module;
use core\{database\Database, utility\Utility, theme\Theme, route\Route, translation\Translation as T};

class Module {
	use Api;

	function admin() {
		if (!isset($_POST['save_config'])) {
			$data = [
				'module_delete' => T::get('Удалить модули'),
				'on' => T::get('Включено'),
				'name_phrase' => T::get('Название'),
				'version' => T::get('Версия'),
				'description' => T::get('Описание'),
				'dependencies' => T::get('Зависимости'),
				'save' => T::get('Подтвердить'),
				
				'delete_route' => Route::render_route('admin/module/delete'),
			];

			$modules = array_merge(glob(EQSASH_ROOT.'/modules/*/*.ini'), glob(EQSASH_ROOT.'/custom/modules/*/*.ini'));
			foreach ($modules as $file) {
				$ini_array = parse_ini_file($file);
				$type = mb_strpos($file, 'custom') === false ? 'base' : 'custom';

				$dependencies = $ini_array['dependencies'] ?? '';
				$disabled = $this->dependencies($ini_array);

				$result_module = Database::select('module')
					->data(['status'])
					->where('name',  $ini_array['name'])
					->fetch('field');

				$checked = $result_module == 1 ? ' checked ' : ' ';

				$data['items'][] = [
					'name' => $ini_array['name'],
					'version' => $ini_array['version'],
					'description' => $ini_array['description'],
					'dependencies' => $dependencies,
					'checked' => $checked,
					'disabled' => $disabled,
					'type' => $type,
				];
			}

			$result = (new Theme)->get_template(__DIR__, 'admin', $data);

			return $result;
		} else {
			$this->save_config();
		}
	}

	function dependencies($ini_array) {
		$disabled = '';
		if (isset($ini_array['dependencies'])) {
			$dependencies = explode(', ', $ini_array['dependencies']);

			foreach ($dependencies as $depend) {
				$result_module = Database::select('module')
					->data(['status'])
					->where('name',  $depend)
					->fetch('object');

				if (!$result_module || ($result_module && $result_module->status == 0)) {
					Database::update('module')
						->data(['status' => 0])
						->where('name',  $ini_array['name'])
						->execute();

					$disabled = ' disabled';
				}
			}
		}

		return $disabled;
	}

	function save_config() {
		unset($_POST['save_config']);

		$types = [];
		foreach ($_POST as $key => $value) {
			if (mb_strpos($key, '_type') !== false) {
				$name = mb_strcut($key, 0, -5);
				unset($_POST[$key]);
				$types[$name] = $value;
			}
		}

		$result_module = Database::select('module')
			->data(['name'])
			->where('status',  1)
			->fetch('assoc_all');

		if ($result_module) {
			foreach ($result_module as $module) {
				$array = array_keys($_POST);

				if (array_search($module['name'], $array) === false) {
					$this->run_disable($module['name'], $types[$module['name']]);
				}
			}
		}

		if ($_POST) {
			$array = array_keys($_POST);
			foreach ($array as $name) {
				$result_module = Database::select('module')
					->data(['name', 'status'])
					->where('name',  $name)
					->fetch('assoc');

				if ($result_module) {
					if (!$result_module['status']) {
						$this->run_enable($name, $types[$name]);
					}
				} else {
					$this->run_install($name, $types[$name]);
				}
			}
		}

		Utility::set_message('Настройки сохранены', 'green');
		unset($_POST);
		Route::go_to('admin/module');
	}

	function delete() {
		$result = '';
		if (!$_POST || isset($_POST['cancel'])) {
			$data = [
				'delete_empty' => T::get('Удалить можно только ранее установленные и выключенные модули'),
				'module_list' => T::get('Вернуться к списку модулей'),
				'delete' => T::get('Удалить'),
				
				'admin_route' => Route::render_route('admin/module'),
			];

			$modules = Database::select('module')
				->where('status',  0)
				->fetch('assoc_all');

			if ($modules) {
				$data['modules'] = $modules;
				$data['name_phrase'] = T::get('Название');
			}

			$result = (new Theme)->get_template(__DIR__, 'delete', $data);
		} else if (isset($_POST['confirm'])) {
			unset($_POST['confirm']);
			$module_delete = implode(',', array_keys($_POST));

			$result = $module_delete ? Utility::get_form_delete('', $module_delete) : Route::go_to('admin/module/delete');
		} else if (isset($_POST['submit'])) {
			unset($_POST['submit']);

			$module_delete = isset($_POST['data']) ? explode(',', $_POST['data']) : [];

			foreach ($module_delete as $name) {
				$module = Database::select('module')
					->where('name',  $name)
					->fetch('assoc');

				$this->run_uninstall($name, $module['type']);
			}

			Utility::set_message('Выбранные модули успешно удалены, теперь их можно удалить с диска', 'green');
			Route::go_to('admin/module');
		}

		return $result;
	}

	//

	function run_install($name, $type) {
		Database::insert('module')
			->data([
				'name' => $name,
				'type' => $type,
				'status' => 1,
			])
			->execute();

		$this->set_operation($name, $type, 'install');
	}

	function run_uninstall($name, $type) {
		$this->set_operation($name, $type, 'uninstall');

		Database::delete('module')
			->where('name', $name)
			->execute();
	}

	function run_enable($name, $type) {
		Database::update('module')
			->data(['status' => 1])
			->where('name',  $name)
			->execute();

		$this->set_operation($name, $type, 'enable');

		Database::update('container')
			->data(['status' => 1])
			->where('entity', $name)
			->execute();

		Database::update('field')
			->data(['status' => 1])
			->where('entity', $name)
			->execute();
	}

	function run_disable($name, $type) {
		$this->set_operation($name, $type, 'disable');

		Database::update('container')
			->data(['status' => 0])
			->where('entity', $name)
			->execute();

		Database::update('field')
			->data(['status' => 0])
			->where('entity', $name)
			->execute();

		Database::update('module')
			->data(['status' => 0])
			->where('name',  $name)
			->execute();
	}

	function set_operation($name, $type, $operation) {
		$path = $type == 'base' ? '' : 'custom';
		$entity = $path.'\modules\\'.$name.'\\Service';

		if (method_exists($entity, $operation)) {
			(new $entity)->{$operation}();
		}

		Utility::clear_cache();
	}
}
