<?php

namespace core\route;
use core\{database\Database, utility\Utility, theme\Theme, user\User, form\Form, field\Field, content\Content, module\Module, application\Application, translation\Translation as T};

class Route {
	use Api, Data;
	
	function get_api_external() {
		$data = file_get_contents('php://input');
		$_POST['data'] = $_POST['data'] ?? $data;
		
		$module = $_GET['module'] ?? $_POST['module'] ?? '';
		$method = $_GET['method'] ?? $_POST['method'] ?? '';
		
		$result = '';
		if ($module && $method) {
			$entities = self::get_entities();
			$entity = $entities[$module] ?? '';
			$entity_service = $entity ? rtrim($entity, $module).'Service' : '';
			
			if (method_exists($entity_service, 'get_api')) {
				$api = (new $entity_service)->get_api();
				
				if (isset($api[$method])) {
					$user = new User;
					$api_method = $api[$method];
					$type = $api_method['type'] ?? 'internal';
					$access = $api_method['access'] ?? [];
					$format = $api_method['format'] ?? '';
					$method = $api_method['method'] ?? $method;
					
					if ($type == 'external' && (!$access || ($user->check_authentication() && $user->get_access($access)))) {
						$data = $_GET['data'] ?? $_POST['data'] ?? '';
						$data = $data ? Utility::decode_json($data) : $_POST;
						
						$method_data = new \ReflectionMethod($entity, $method);
						$result = $method_data->isStatic() ? $entity::$method($data) : (new $entity)->$method($data);
						$result = $format == 'json' || $format == 'json_object' ? Utility::encode_json($result, $format == 'json_object') : $result;
					}
				}
			}
		}
		
		return $result;
	}

	static function set_uri_referer() {
		if (self::get_route(true, 0) != 'api' && mb_strpos(self::get_route(), '.') === false) {
			$_SESSION['HTTP_REFERER'] = self::get_route(false, null, false);
		}
	}

	static function get_uri_referer($default = true) {
		$uri = $default ? $_SERVER['HTTP_REFERER'] : ($_SESSION['HTTP_REFERER'] ?? '');

		return $uri;
	}

	static function get_route($explode = false, $number = null, $source = true, $alias = false) {
		$route = Api::parse_route();

		if (Database::check()) {
			if ($source) {
				$route_aliases = Database::select('route_aliases')
					->data(['entity_id', 'type'])
					->where('alias', $route)
					->fetch('assoc');

				$route = $route_aliases ? $route_aliases['type'].'/'.$route_aliases['entity_id'] : $route;
			}
			if ($alias) {
				$route_parts = Utility::get_string_parts($route, '/');

				$route = Database::select('route_aliases')
					->data(['alias'])
					->where('entity_id', $route_parts[1] ?? 0)
					->where('type', $route_parts[0] ?? '')
					->fetch('field');
			}
		}

		$route = $explode ? Utility::get_string_parts($route, '/', $number) : $route;

		return $route;
	}

	static function get_route_id($number = -1) {
		$route_id = self::get_route(true, $number);

		return $route_id;
	}

	static function get_path($path, $route = true, $explode = false, $number = null) {
		$clear = !$route && !$explode ? 'rtrim' : 'trim';
		
		$path = $route ? str_replace(EQSASH_ROOT, '', $path) : $path;
		$path = $clear(str_replace('\\', '/', $path), '/');

		$path = $explode ? Utility::get_string_parts($path, '/', $number) : $path;

		return $path;
	}

	static function search_entities_standard($type = '*', $property = 'namespace') {
		$entities = Utility::get_variable('entities_standard');

		if (!$entities) {
			$entities = [];

			$entities_standard = glob(EQSASH_ROOT.'/'.$type.'/*/*.php');
			foreach ($entities_standard as $key => $value) {
				$name_full = basename($value);
				$name = mb_substr($name_full, 0, -4);

				$path = rtrim(rtrim($value, $name_full), '/');
				$path_parts = self::get_path($path, false, true);
				$name_entity = end($path_parts);

				$namespace_prefix = $path_parts[count($path_parts) - 2];
				$namespace = $namespace_prefix.'\\'.$name.'\\'.$name;

				if ($name == $name_entity) {
					$entities[$namespace_prefix][$name] = ${$property};
				}
			}

			Utility::set_variable('entities_standard', $entities);
		}

		return $entities;
	}

	static function get_entities($entity = '', $property = 'namespace', $type = '', $database = true) {
		$entities = self::search_entities_standard('*', $property);

		if ($database) {
			$entities_database = [];

			$entities_module = (new Module)->get_list();
			$entities_theme = (new Theme)->get_list();

			$entities_other = array_merge($entities_module, $entities_theme);
			foreach ($entities_other as $key => $value) {
				$entities_types = isset($value->regions) ? 'themes' : 'modules';

				$namespace_prefix = $value->type == 'base' ? $entities_types : 'custom\\'.$entities_types;
				$namespace = $namespace_prefix.'\\'.$value->name.'\\'.$value->name;

				$path_prefix = $value->type == 'base' ? $entities_types : '/custom/'.$entities_types;
				$path = $path_prefix.'/'.$value->name;

				$entities_database[$value->type][$value->name] = ${$property};
			}

			$entities = [
				'core' => $entities['core'],
				'base' => $entities_database['base'] ?? [],
				'custom' => $entities_database['custom'] ?? [],
			];
		} else {
			$entities = [
				'core' => $entities['core'],
				'base' => array_merge($entities['modules'], $entities['themes']),
				'custom' => [],
			];
		}

		$entities_all = $entities['core'] + $entities['base'] + $entities['custom'];
		$entities = $type ? ($entities[$type] ?? $entities_all) : $entities_all;
		$entities = $entity ? ($entities[$entity] ?? '') : $entities;

		return $entities;
	}

	static function get_routes_all($entities = []) {
		$entities = $entities ?: self::get_entities();

		$routes = [];
		$cache = Utility::check_cache();

		$routes_file = EQSASH_ROOT.'/'.Route::$path_cache.'/routes.php';
		if ($cache && file_exists($routes_file)) {
			require $routes_file;
		} else {
			foreach ($entities as $key => $value) {
				$entity = rtrim($value, $key).'Service';

				if (method_exists($entity, 'get_routes')) {
					$routes[$value] = (new $entity)->get_routes();
				}
			}

			if ($cache) {
				foreach ($routes as $key => $value) {
					foreach ($value as $key_two => $value_two) {
						if (isset($value_two['title'])) {
							$value_two['title'] = T::get($value_two['title']);
							$value[$key_two] = $value_two;
						}
					}

					$routes[$key] = $value;
				}

				$routes_content = '<?php'.PHP_EOL.PHP_EOL.'$routes = '.var_export($routes, true).';'.PHP_EOL;
				file_put_contents($routes_file, $routes_content);
			}
		}

		return $routes;
	}

	static function get_routes($uri = '', $mask = '', $pattern = '[0-9a-z\.\-]+') {
		$entities = self::get_entities();

		$routes = self::get_routes_all($entities);
		$routes_filter = [];

		foreach ($entities as $key => $value) {
			$entity_type = Utility::get_string_parts($value, '\\', 0);
			$entity_type = $entity_type == 'core' ? 'core' : ($entity_type == 'custom' ? 'custom' : 'base');
			$entity_path = '/'.self::get_path(rtrim(rtrim($value, $key), '\\'));

			$routes_item = $routes[$value] ?? [];
			foreach ($routes_item as $key_two => $value_two) {
				$uri_temp = $uri ?: $key_two;
				$mask_temp = $mask ?: $key_two;
				$replace = $mask_temp == '*' ? '[0-9a-z\.\/\-]+' : $pattern;
				$pattern_temp = '/^'.str_replace('*', $replace, addcslashes($mask_temp, '/')).'$/';

				$uri_temp_parts = Utility::get_string_parts($uri_temp, '/');
				array_pop($uri_temp_parts);
				$parent = implode('/', $uri_temp_parts);

				if (preg_match($pattern_temp, $uri_temp) || (!$uri && !$mask)) {
					$value_two['entity'] = $key;
					$value_two['entity_namespace'] = $value;
					$value_two['entity_path'] = $entity_path;
					$value_two['entity_type'] = $entity_type;
					$value_two['route'] = self::render_route($uri_temp);
					$value_two['route_original'] = self::render_route($key_two);
					$value_two['parent'] = self::render_route($parent);

					$routes_filter[] = $value_two;
				}
			}
		}

		uasort($routes_filter, function ($one, $two) {
			$one_item = $one['weight'] ?? 0;
			$two_item = $two['weight'] ?? 0;

			return $one_item > $two_item ? 1 : ($one_item < $two_item ? -1 : 0);
		});

		$routes = $routes_filter;

		return $routes;
	}

	static function get_route_language_param($symbol = '?', $language = '') {
		$language = $language ?: T::get_language();
		$param = $language == T::get_language_default() ? '' : ($language ? $symbol.'l='.$language : '');

		return $param;
	}

	static function get_route_alias($id, $type = 'content', $full = false) {
		$fetch = 'object';

		$route_alias = Database::select('route_aliases');

		if (!$full) {
			$fetch = 'field';
			$route_alias->data(['alias']);
		}

		$route_alias = $route_alias->where('entity_id', $id)
			->where('type', $type)
			->fetch($fetch);

		return $route_alias;
	}

	static function make_route_alias($type, $title, $multi = true) {
		$content = new Content;

		$type_name = $content->get_type_name($type);
		$type_name .= $multi ? 's' : '';

		$title_translit = T::get_translit($title);
		$title_translit_parts = explode('-', $title_translit);

		$title_translit_parts_new = [];
		foreach ($title_translit_parts as $title_translit_part) {
			if ($title_translit_part !== '') {
				$title_translit_parts_new[] = $title_translit_part;
			}
		}
		$title_translit = implode('-', $title_translit_parts_new);

		$result = $type_name.'/'.$title_translit;

		return $result;
	}

	//

	static function go($uri = '', $die = false) {
		$location = Api::get_host().self::render_route(trim($uri, '/'));
		header('Location: '.$location);

		if ($die) {
			die;
		}
	}

	static function go_to($value = '') {
		$GLOBALS['eqsash']['go_to'] = $value;
	}

	static function get_go_to() {
		$go_to = $GLOBALS['eqsash']['go_to'] ?? false;

		return $go_to;
	}

	static function check_page_home() {
		$home = self::get_route(false, null, false) == '/';

		return $home;
	}

	static function get_path_public() {
		$directory = self::get_path(EQSASH_ROOT, false).'/custom/files';

		return $directory;
	}

	static function get_path_private() {
		$directory = self::get_path(EQSASH_ROOT, false).'/custom/files/private';

		return $directory;
	}

	static function check_mode($mode) {
		$mode = self::get_route(true, 0) == $mode;

		return $mode;
	}
	
	static function get_pager($total, $limit, $link, $delimiter = ' ', $all = false) {		
		$route_param = self::get_route_language_param('&');
		
		$pages_count = ceil($total / $limit);
		
		$page = $_GET['page'] ?? 1;
		$page = (is_numeric($page) && $page > 0 && $page <= $pages_count) || ($all && $page == 'all') ? $page : 1;
		
		$pages = range(1, $pages_count) + ($all ? ['all' => 'Все',] : []);

		$html = '<div>';
		foreach ($pages as $key => $value) {
			$key_temp = is_numeric($key) ? $key + 1 : $key;
			
			$active = $page == $key_temp ? ' class="link-active"' : '';
			$html .= '<a href="'.$link.'?page='.$key_temp.$route_param.'"'.$active.'>'.$value.'</a>'.$delimiter;
		}
		$html = rtrim($html, $delimiter);
		$html .= '</div>';
		
		$offset = is_numeric($page) ? ($page - 1) * $limit : 'all';
		
		$pager = compact('offset', 'html', 'page');
		
		return $pager;
	}
	
	static function set_pager_title($page) {	
		$title = Application::get_title();
		$title = is_numeric($page) && $page > 1 ? $title .' ('.T::get('стр').'. '.$page.')' : $title;
		Application::set_title($title);
	}

	static function render_route($route, $slash = true, $reverse = false, $language = []) {
		$slash = $slash ? '/' : '';

		if (EQSASH_INTERIM) {
			$route_clean = ltrim($route, '/\\');
			
			if ($route_clean) {
				$route = $reverse ? $route_clean.'/'.EQSASH_INTERIM : EQSASH_INTERIM.'/'.$route_clean;
			} else {
				$route = EQSASH_INTERIM;
			}
		}
		
		if ($language) {
			$language_current = T::get_language();
			$parameter = self::get_route_language_param($language['symbol'] ?? '?', $language['language'] ?? $language_current);
			$route .= $parameter && mb_strpos($route, $parameter) === false ? $parameter : '';
		}
		
		$route = $slash.$route;

		return $route;
	}

	static function get_interim_count() {
		$count = EQSASH_INTERIM ? count(explode('/', EQSASH_INTERIM)) : 0;

		return $count;
	}

	static function get_interim_token() {
		$token = EQSASH_INTERIM ? '/'.EQSASH_INTERIM : '';
		
		return $token;
	}

	static function update_interim_files() {
		$files = [
			'others/fonts/opensans/style.css' => 'others/templates/style.css',
		];
		
		$token = self::get_interim_token();
		
		foreach ($files as $path => $template) {
			$content_template = file_get_contents(EQSASH_ROOT.'/'.$template);
			$content = str_replace('[INTERIM]', $token, $content_template);
			file_put_contents(EQSASH_ROOT.'/'.$path, $content);
		}
	}

	static function check_api() {
		$key = $_GET['key'] ?? $_POST['key'] ?? '';
		$result = $key && $key == Utility::get_settings('api_key');

		return $result;
	}
}
